import pytest

from holodex.client import HolodexClient


@pytest.fixture()
async def client():
    client = HolodexClient()
    yield client
    if client.session:
        await client.session.close()



from typing import Any, Union
from typing_extensions import Literal


class LiteChannel:
    _response: Any

    def __init__(self, response: Any) -> None:
        self._response: Any = response

    def __repr__(self):
        return f'LiteChannel("{self.id}", "{self.name}")'

    @property
    def id(self) -> str:
        return self._response.get("id")

    @property
    def name(self) -> str:
        return self._response.get("name")

    @property
    def english_name(self) -> str:
        return self._response.get("english_name")

    @property
    def type(self) -> Union[Literal["vtuber"], str]:
        return self._response.get("type")

    @property
    def org(self):
        return self._response.get("org")

    @property
    def suborg(self):
        return self._response.get("suborg")

    @property
    def group(self) -> str:
        return self._response.get("group")

    @property
    def photo(self) -> str:
        return self._response.get("photo")

    @property
    def twitter(self) -> str:
        return self._response.get("twitter")

    @property
    def video_count(self) -> int:
        return int(self._response.get("video_count"))

    @property
    def subscriber_count(self) -> int:
        return int(self._response.get("subscriber_count"))

    @property
    def clip_count(self) -> int:
        return self._response.get("clip_count")

    @property
    def top_topics(self) -> list[str]:
        return self._response.get("top_topics")


class Channels:
    _response: list[Any]

    def __init__(self, response: list[Any]) -> None:
        if type(response) == list:
            self._response = [LiteChannel(channel) for channel in response]
        else:
            self._response = response

    def __repr__(self):
        return f"Channels({len(self)})"

    def __getitem__(self, index: Union[slice, int]) -> Union[LiteChannel, Any]:
        return self._response.__getitem__(index)

    def __len__(self):
        return len(self._response)



from holodex.model.channel_video import ChannelVideo
from holodex.model.channels import Channels
from holodex.model.video import Video
from holodex.model.live import Live
from holodex.model.autocomplete import AutoComplete
from holodex.model.search_video import SearchVideo
from typing import Any, Literal, Optional
from aiohttp.client import ClientSession

from holodex.http import HolodexHttpClient
from holodex.model.channel import Channel


class HolodexClient(HolodexHttpClient):
    def __init__(
        self, key: Optional[str] = None, session: Optional[ClientSession] = None
    ) -> None:
        super().__init__(key, session)

    def __get_params(
        self, keys: dict[str, Any], exclude: Optional[list[str]] = None
    ) -> dict[str, Any]:
        keys.pop("self")
        if exclude:
            for key in exclude:
                keys.pop(key)
        return {k: v for k, v in keys.items() if v is not None}

    async def channel(self, channel_id: str) -> Channel:
        return Channel(await self.get_channel(channel_id))

    async def channels(
        self,
        *,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        type: Optional[Literal["vtuber"]] = None,
        order: Optional[Literal["asc", "desc"]] = None,
        org: Optional[
            Literal["All Vtubers", "Hololive", "Nijisanji", "Independents"]
        ] = None,
        sort: Optional[str] = None
    ) -> Channels:
        params = self.__get_params(locals())
        return Channels(await self.get_channels(**params))

    async def autocomplete(self, keyword: str) -> AutoComplete:
        return AutoComplete(await self.get_autocomplete(keyword))

    async def live_streams(
        self,
        *,
        channel_id: Optional[str] = None,
        id: Optional[str] = None,
        include: Optional[list[str]] = None,
        lang: Optional[Literal["all", "en", "ja"]] = None,
        limit: Optional[int] = None,
        max_upcoming_hours: Optional[int] = None,
        mentioned_channel_id: Optional[str] = None,
        offset: Optional[int] = None,
        order: Optional[Literal["asc", "desc"]] = None,
        org: Optional[
            Literal["All Vtubers", "Hololive", "Nijisanji", "Independents"]
        ] = None,
        paginated: Optional[str] = None,
        sort: Optional[str] = None,
        status: Optional[str] = None,
        topic: Optional[str] = None,
        type: Optional[str] = None
    ) -> Live:
        params = self.__get_params(locals())
        return Live(await self.get_live_streams(params))

    async def video(self, video_id: str) -> Video:
        params = {"c": 1, "lang": "all"}
        return Video(await self.get_video(video_id, params))

    async def videos_from_channel(
        self,
        channel_id: str,
        type: Literal["clips", "videos", "collabs"],
        include: Optional[list[str]] = None,
        lang: Optional[Literal["all", "en", "ja"]] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        paginated: Optional[str] = None,
    ) -> ChannelVideo:
        params = self.__get_params(locals(), ["channel_id", "type"])
        return ChannelVideo(
            await self.get_videos_from_channel(channel_id, type, params)
        )

    async def search_video(
        self,
        offset: int,
        limit: int,
        sort: Literal["oldest", "newest"] = "newest",
        lang: Optional[Literal["all", "en", "ja"]] = None,
        target: Optional[Literal["clip", "stream"]] = None,
        conditions: Optional[list[dict[Literal["text"], str]]] = None,
        topic: Optional[list[str]] = None,
        vch: Optional[list[str]] = None,
        org: Optional[list[str]] = None,
    ) -> SearchVideo:
        params = self.__get_params(locals(), ["offset", "limit", "sort"])
        return SearchVideo(await self.get_search_video(params))



from typing import Any, Literal, Optional


class Channel:
    def __init__(self, response: Any) -> None:
        self._response = response

    @property
    def id(self) -> str:
        return self._response["id"]

    @property
    def name(self) -> str:
        return self._response["name"]

    @property
    def english_name(self) -> Optional[str]:
        return self._response.get("english_name")

    @property
    def description(self) -> Optional[str]:
        return self._response.get("description")

    @property
    def photo(self) -> Optional[str]:
        return self._response.get("photo")

    @property
    def thumbnail(self) -> Optional[str]:
        return self._response.get("thumbnail")

    @property
    def banner(self) -> Optional[str]:
        return self._response.get("banner")

    @property
    def org(self) -> Optional[str]:
        return self._response.get("org")

    @property
    def suborg(self) -> Optional[str]:
        return self._response.get("suborg")

    @property
    def lang(self) -> Optional[str]:
        return self._response.get("lang")

    @property
    def published_at(self) -> Optional[str]:
        return self._response.get("published_at")

    @property
    def view_count(self) -> Optional[str]:
        return self._response.get("view_count")

    @property
    def video_count(self) -> Optional[str]:
        return self._response.get("video_count")

    @property
    def subscriber_count(self) -> Optional[str]:
        return self._response.get("subscriber_count")

    @property
    def comments_crawled_at(self) -> Optional[str]:
        return self._response.get("comments_crawled_at")

    @property
    def updated_at(self) -> Optional[str]:
        return self._response.get("updated_at")

    @property
    def yt_uploads_id(self) -> Optional[str]:
        return self._response.get("yt_uploads_id")

    @property
    def crawled_at(self) -> Optional[str]:
        return self._response.get("crawled_at")

    @property
    def type(self) -> Optional[Literal["vtuber", "subber"]]:
        return self._response.get("type")

    @property
    def clip_count(self) -> Optional[int]:
        return self._response.get("clip_count")

    @property
    def twitter(self) -> Optional[str]:
        return self._response.get("twitter")

    @property
    def inactive(self) -> Optional[bool]:
        return self._response.get("inactive")

    @property
    def created_at(self) -> Optional[str]:
        return self._response.get("created_at")

    @property
    def top_topics(self) -> Optional[list[str]]:
        return self._response.get("top_topics")



from aiohttp import ClientSession
from typing import Any, Literal, Optional
from types import TracebackType


class HolodexHttpClient:
    BASE_URL = "https://holodex.net/api/v2"

    def __init__(
        self, key: Optional[str] = None, session: Optional[ClientSession] = None
    ) -> None:
        self.session = session
        self.key = key

    async def close(self) -> None:
        if self.session:
            await self.session.close()

    async def __aenter__(self) -> "HolodexHttpClient":
        return self

    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        await self.close()

    @property
    def headers(self) -> dict[str, Any]:
        headers: dict[str, Any] = {}
        if self.key:
            headers["X-APIKEY"] = self.key
        return headers

    async def request(
        self,
        method: Literal["GET", "POST"],
        endpoint: str,
        **kwargs: Any,
    ) -> Any:
        if not self.session:
            self.session = ClientSession()

        async with self.session.request(
            method,
            self.BASE_URL + endpoint,
            headers=self.headers,
            **kwargs,
        ) as r:
            return await r.json()

    async def get_channel(self, channel_id: str) -> Any:
        return await self.request("GET", f"/channels/{channel_id}")

    async def get_channels(self, **params: Any) -> Any:
        return await self.request(
            "GET", f"/channels", params={"limit": 100, "offset": 100, **params}
        )

    async def get_autocomplete(self, q: str) -> Any:
        return await self.request("GET", f"/search/autocomplete", params={"q": q})

    async def get_live_streams(self, params: dict[str, Any]) -> Any:
        return await self.request("GET", f"/live", params=params)

    async def get_video(self, video_id: str, params: dict[str, Any]) -> Any:
        return await self.request("GET", f"/videos/{video_id}", params=params)

    async def get_videos_from_channel(
        self,
        channel_id: str,
        type: Literal["clips", "videos", "collabs"],
        params: dict[str, Any],
    ) -> Any:
        return await self.request(
            "GET", f"/channels/{channel_id}/{type}", params=params
        )

    async def get_search_video(self, params: dict[str, Any]) -> Any:
        return await self.request("POST", "/search/videoSearch", params=params)



import setuptools
from setuptools import setup
import holodex


setup(
    name="holodex",
    version=holodex.__version__,
    author="ombe1229",
    author_email="h3236516@gmail.com",
    description="Holodex api wrapper",
    license="Apache 2.0",
    packages=setuptools.find_packages(),
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/ombe1229/holodex",
)
